@@ -226,12 +226,12 @@ class simulation_model():
     
     def rank(self, lis, rank):
         rank = int(rank)
-        sorted_list = sorted(lis)
+        sorted_list = np.sort(lis)
         try:
             rankth_elem = sorted_list[rank-1]
         except IndexError as e:
             logging.error("RANK: Rank {} too high for array of size {}".format(rank,len(lis)))
-        return lis.index(rankth_elem)+1
+        return (lis==rankth_elem).nonzero()[0][0]+1
         
 
     def interpolate(self, variable, t, *args):
@@ -586,8 +586,8 @@ class simulation_model():
         for product in products:
             prod = str(product).replace("(", "").replace(")", "").replace("[", "").replace("]", "").replace("'", "").replace(" ", "")
             return_list += [self.memoize(equation_basic + "[{}]".format(prod), t)]
-
-        return return_list
+            
+        return np.array(return_list)
 
 
     #Access equations API
